import { createSlice } from '@reduxjs/toolkit';

import {
  Sha256,
  Sha512,
  string_to_bytes,
  bytes_to_base64,
} from '@openpgp/asmcrypto.js';

interface TracksState {
  tracks: { [index: string]: any };
}

const initialTracks: TracksState = {
  tracks: {},
};

const tracksSlice = createSlice({
  name: 'tracks',
  initialState: initialTracks,
  reducers: {
    add: (state, action) => {
      const { id, ...data } = action.payload;
      state.tracks[action.payload.id] = data;
    },
    push: (state, action) => {
      const sha = new Sha512();
      const result = sha
        .process(string_to_bytes(JSON.stringify(action.payload.track)))
        .finish().result;
      var id;
      if (result === null) {
        console.log(`Can't hash`);
        id = crypto.randomUUID();
      } else {
        id = bytes_to_base64(result);
        console.log(`Digest: ${id}`);
      }
      if (state.tracks.hasOwnProperty(id)) {
        console.log('Track had already been imported');
      } else {
        state.tracks[id] = {
          ...action.payload,
        };
      }
    },
  },
});

export const tracksActions = tracksSlice.actions;

export default tracksSlice.reducer;
